@@ -1,7 +1,7 @@
 //---------------------------------------------------------------------------------
 //
 //  Little Color Management System
-//  Copyright (c) 1998-2011 Marti Maria Saguer
+//  Copyright (c) 1998-2022 Marti Maria Saguer
 //
 // Permission is hereby granted, free of charge, to any person obtaining 
 // a copy of this software and associated documentation files (the "Software"), 
@@ -96,43 +96,44 @@ void Help(void)
     fprintf(stderr, "usage: transicc [flags] [CGATS input] [CGATS output]\n\n");
 
     fprintf(stderr, "flags:\n\n");
-    fprintf(stderr, "%cv<0..3> - Verbosity level\n", SW); 
+    fprintf(stderr, "-v<0..3> - Verbosity level\n"); 
 
-    fprintf(stderr, "%ce[op] - Encoded representation of numbers\n", SW);
-    fprintf(stderr, "\t%cw - use 16 bits\n", SW);     
-    fprintf(stderr, "\t%cx - Hexadecimal\n\n", SW);
+    fprintf(stderr, "-e[op] - Encoded representation of numbers\n");
+    fprintf(stderr, "\t-w - use 16 bits\n");     
+    fprintf(stderr, "\t-x - Hexadecimal\n\n");
 
-    fprintf(stderr, "%cs - bounded mode (clip negatives and highliths)\n", SW);
-    fprintf(stderr, "%cq - Quantize (round decimals)\n\n", SW);
+    fprintf(stderr, "-s - bounded mode (clip negatives and highlights)\n");
+    fprintf(stderr, "-q - Quantize (round decimals)\n\n");
 
-    fprintf(stderr, "%ci<profile> - Input profile (defaults to sRGB)\n", SW);
-    fprintf(stderr, "%co<profile> - Output profile (defaults to sRGB)\n", SW);   
-    fprintf(stderr, "%cl<profile> - Transform by device-link profile\n", SW);   
-
-    fprintf(stderr, "\nYou can use '*Lab', '*xyz' and others as built-in profiles\n\n");
+    fprintf(stderr, "-i<profile> - Input profile (defaults to sRGB)\n");
+    fprintf(stderr, "-o<profile> - Output profile (defaults to sRGB)\n");   
+    fprintf(stderr, "-l<profile> - Transform by device-link profile\n");   
+   
+    PrintBuiltins();
 
     PrintRenderingIntents();
 
     fprintf(stderr, "\n");
 
-    fprintf(stderr, "%cd<0..1> - Observer adaptation state (abs.col. only)\n\n", SW);
+    fprintf(stderr, "-d<0..1> - Observer adaptation state (abs.col. only)\n\n");
 
-    fprintf(stderr, "%cb - Black point compensation\n", SW);
+    fprintf(stderr, "-b - Black point compensation\n");
 
-    fprintf(stderr, "%cc<0,1,2,3> Precalculates transform (0=Off, 1=Normal, 2=Hi-res, 3=LoRes)\n\n", SW);     
-    fprintf(stderr, "%cn - Terse output, intended for pipe usage\n", SW);
+    fprintf(stderr, "-c<0,1,2,3> Precalculates transform (0=Off, 1=Normal, 2=Hi-res, 3=LoRes)\n\n");     
+    fprintf(stderr, "-n - Terse output, intended for pipe usage\n");
 
-    fprintf(stderr, "%cp<profile> - Soft proof profile\n", SW);
-    fprintf(stderr, "%cm<0,1,2,3> - Soft proof intent\n", SW);
-    fprintf(stderr, "%cg - Marks out-of-gamut colors on softproof\n\n", SW);
+    fprintf(stderr, "-p<profile> - Soft proof profile\n");
+    fprintf(stderr, "-m<0,1,2,3> - Soft proof intent\n");
+    fprintf(stderr, "-g - Marks out-of-gamut colors on softproof\n\n");
 
 
 
-    fprintf(stderr, "This program is intended to be a demo of the little cms\n"
-        "engine. Both lcms and this program are freeware. You can\n"
-        "obtain both in source code at http://www.littlecms.com\n"
+    fprintf(stderr, "This program is intended to be a demo of the Little CMS\n"
+        "color engine. Both lcms and this program are open source.\n"
+        "You can obtain both in source code at https://www.littlecms.com\n"
         "For suggestions, comments, bug reports etc. send mail to\n"
         "info@littlecms.com\n\n");
+
 }
 
 
@@ -145,10 +146,22 @@ void HandleSwitches(int argc, char *argv[])
     int s;
 
     while ((s = xgetopt(argc, argv,
-        "bBC:c:d:D:eEgGI:i:L:l:m:M:nNO:o:p:P:QqSsT:t:V:v:WwxX!:")) != EOF) {
+        "bBC:c:d:D:eEgGI:i:L:l:m:M:nNO:o:p:P:QqSsT:t:V:v:WwxX!:-:")) != EOF) {
 
     switch (s){
 
+        case '-':
+            if (strcmp(xoptarg, "help") == 0)
+            {
+                Help();
+                exit(0);
+            }
+            else
+            {
+                FatalError("Unknown option - run without args to see valid ones.\n");
+            }
+            break;
+            
         case '!': 
             IncludePart = xoptarg;
             break;
@@ -386,7 +399,7 @@ cmsNAMEDCOLORLIST* ComponentNames(cmsColorSpaceSignature space, cmsBool IsInput)
 
         SetRange(1, IsInput);
 
-        n = cmsChannelsOf(space);
+        n = cmsChannelsOfColorSpace(space);
 
         for (i=0; i < n; i++) {
 
@@ -457,7 +470,7 @@ cmsBool OpenTransforms(void)
 
         if (cmsGetDeviceClass(hInput) == cmsSigLinkClass ||
             cmsGetDeviceClass(hOutput) == cmsSigLinkClass)   
-            FatalError("Use %cl flag for devicelink profiles!\n", SW);
+            FatalError("Use -l flag for devicelink profiles!\n");
 
 
         InputColorSpace   = cmsGetColorSpace(hInput);
@@ -529,7 +542,7 @@ cmsBool OpenTransforms(void)
 
     // For named color, there is a specialized formatter
     if (cmsGetDeviceClass(hInput) == cmsSigNamedColorClass) {
-        dwOut = dwIn;
+        
         dwIn = TYPE_NAMED_COLOR_INDEX;
         InputNamedColor = TRUE;
     }
@@ -652,7 +665,7 @@ void PrintFloatResults(cmsFloat64Number Value[])
     char ChannelName[cmsMAX_PATH];
     cmsFloat64Number v;
 
-    n = cmsChannelsOf(OutputColorSpace);
+    n = cmsChannelsOfColorSpace(OutputColorSpace);
     for (i=0; i < n; i++) {
 
         if (OutputColorant != NULL) {
@@ -691,7 +704,7 @@ void PrintFloatResults(cmsFloat64Number Value[])
 static
 cmsUInt16Number GetIndex(void)
 {
-    char Buffer[4096], Name[40], Prefix[40], Suffix[40];
+    char Buffer[4096], Name[cmsMAX_PATH], Prefix[40], Suffix[40];
     int index, max;
     const cmsNAMEDCOLORLIST* NamedColorList;
     
@@ -719,7 +732,7 @@ void TakeFloatValues(cmsFloat64Number Float[])
 {
     cmsUInt32Number i, n;
     char ChannelName[cmsMAX_PATH];
-    char Buffer[cmsMAX_PATH];
+    char Buffer[4096];
 
     if (xisatty(stdin))
         fprintf(stderr, "\nEnter values, 'q' to quit\n");
@@ -732,7 +745,7 @@ void TakeFloatValues(cmsFloat64Number Float[])
         return;
     }
 
-    n = cmsChannelsOf(InputColorSpace);
+    n = cmsChannelsOfColorSpace(InputColorSpace);
     for (i=0; i < n; i++) {
 
         if (InputColorant) {
@@ -781,7 +794,7 @@ void PrintEncodedResults(cmsUInt16Number Encoded[])
     char ChannelName[cmsMAX_PATH];
     cmsUInt32Number v;
 
-    n = cmsChannelsOf(OutputColorSpace);
+    n = cmsChannelsOfColorSpace(OutputColorSpace);
     for (i=0; i < n; i++) {
 
         if (OutputColorant != NULL) {
@@ -858,7 +871,7 @@ cmsFloat64Number GetIT8Val(const char* Name, cmsFloat64Number Max)
 // Read input values from CGATS file.
 
 static
-    void TakeCGATSValues(int nPatch, cmsFloat64Number Float[])
+void TakeCGATSValues(int nPatch, cmsFloat64Number Float[])
 {
 
     // At first take the name if SAMPLE_ID is present
@@ -947,7 +960,7 @@ static
         {
             cmsUInt32Number i, n;
 
-            n = cmsChannelsOf(InputColorSpace);
+            n = cmsChannelsOfColorSpace(InputColorSpace);
             for (i=0; i < n; i++) { 
 
                 char Buffer[255];
@@ -963,7 +976,7 @@ static
         {
             cmsUInt32Number i, n;
 
-            n = cmsChannelsOf(InputColorSpace);
+            n = cmsChannelsOfColorSpace(InputColorSpace);
             for (i=0; i < n; i++) { 
 
                 char Buffer[255];
@@ -1054,9 +1067,9 @@ void PutCGATSValues(cmsFloat64Number Float[])
     case cmsSig15colorData:
         {
 
-            cmsUInt32Number i, n;
+            cmsInt32Number i, n;
 
-            n = cmsChannelsOf(InputColorSpace);
+            n = cmsChannelsOfColorSpace(InputColorSpace);
             for (i=0; i < n; i++) { 
 
                 char Buffer[255];
@@ -1071,9 +1084,9 @@ void PutCGATSValues(cmsFloat64Number Float[])
     default: 
         {
 
-            cmsUInt32Number i, n;
+            cmsInt32Number i, n;
 
-            n = cmsChannelsOf(InputColorSpace);
+            n = cmsChannelsOfColorSpace(InputColorSpace);
             for (i=0; i < n; i++) { 
 
                 char Buffer[255];
@@ -1174,7 +1187,7 @@ void SetOutputDataFormat(void)
             int i, n;
             char Buffer[255];
 
-            n = cmsChannelsOf(OutputColorSpace);
+            n = cmsChannelsOfColorSpace(OutputColorSpace);
             cmsIT8SetPropertyDbl(hIT8out, "NUMBER_OF_FIELDS", n+1);
             cmsIT8SetDataFormat(hIT8out, 0, "SAMPLE_ID");
 
@@ -1190,7 +1203,7 @@ void SetOutputDataFormat(void)
         int i, n;
         char Buffer[255];
 
-        n = cmsChannelsOf(OutputColorSpace);
+        n = cmsChannelsOfColorSpace(OutputColorSpace);
         cmsIT8SetPropertyDbl(hIT8out, "NUMBER_OF_FIELDS", n+1);
         cmsIT8SetDataFormat(hIT8out, 0, "SAMPLE_ID");
 
@@ -1240,7 +1253,9 @@ int main(int argc, char *argv[])
 
     int nPatch = 0;
 
-    fprintf(stderr, "LittleCMS ColorSpace conversion calculator - 4.3 [LittleCMS %2.2f]\n", LCMS_VERSION / 1000.0);
+    fprintf(stderr, "LittleCMS ColorSpace conversion calculator - 5.1 [LittleCMS %2.2f]\n", cmsGetEncodedCMMversion() / 1000.0);
+    fprintf(stderr, "Copyright (c) 1998-2022 Marti Maria Saguer. See COPYING file for details.\n");
+    fflush(stderr);
 
     InitUtils("transicc");
 
